diff --git a/dlls/iphlpapi/iphlpapi_main.c b/dlls/iphlpapi/iphlpapi_main.c
index 4d66810..84c58af 100644
--- a/dlls/iphlpapi/iphlpapi_main.c
+++ b/dlls/iphlpapi/iphlpapi_main.c
@@ -64,6 +64,21 @@ WINE_DEFAULT_DEBUG_CHANNEL(iphlpapi);
 #define INADDR_NONE ~0UL
 #endif
 
+typedef void (WINAPI *LPFN_WSASETLASTERROR)(int);// from winsock2.h
+static LPFN_WSASETLASTERROR pWSASetLastError = NULL;
+
+BOOL LoadWinsock(void)
+{
+  HANDLE mod = NULL;
+  if (pWSASetLastError != NULL) return TRUE;
+  mod = LoadLibraryA("ws2_32.dll");
+  if (NULL != mod)
+  {
+    pWSASetLastError = (LPFN_WSASETLASTERROR)GetProcAddress(mod, "WSASetLastError");
+  }
+  return (NULL != pWSASetLastError);
+}
+
 /******************************************************************
  *    AddIPAddress (IPHLPAPI.@)
  *
@@ -145,7 +160,7 @@ static int IpAddrTableSorter(const void *a, const void *b)
 /******************************************************************
  *    AllocateAndGetIpAddrTableFromStack (IPHLPAPI.@)
  *
- * Get interface-to-IP address mapping table. 
+ * Get interface-to-IP address mapping table.
  * Like GetIpAddrTable(), but allocate the returned table from heap.
  *
  * PARAMS
@@ -251,7 +266,7 @@ DWORD WINAPI CreateIpNetEntry(PMIB_IPNETROW pArpEntry)
  * Create a Proxy ARP (PARP) entry for an IP address.
  *
  * PARAMS
- *  dwAddress [In] IP address for which this computer acts as a proxy. 
+ *  dwAddress [In] IP address for which this computer acts as a proxy.
  *  dwMask    [In] subnet mask for dwAddress
  *  dwIfIndex [In] interface index
  *
@@ -344,7 +359,7 @@ DWORD WINAPI DeleteIpNetEntry(PMIB_IPNETROW pArpEntry)
  * Delete a Proxy ARP entry.
  *
  * PARAMS
- *  dwAddress [In] IP address for which this computer acts as a proxy. 
+ *  dwAddress [In] IP address for which this computer acts as a proxy.
  *  dwMask    [In] subnet mask for dwAddress
  *  dwIfIndex [In] interface index
  *
@@ -476,7 +491,7 @@ DWORD WINAPI GetAdaptersInfo(PIP_ADAPTER_INFO pAdapterInfo, PULONG pOutBufLen)
        * precisely enough space.
        */
       size = sizeof(IP_ADAPTER_INFO) * numNonLoopbackInterfaces;
-      size += numIPAddresses  * sizeof(IP_ADDR_STRING); 
+      size += numIPAddresses  * sizeof(IP_ADDR_STRING);
       if (!pAdapterInfo || *pOutBufLen < size) {
         *pOutBufLen = size;
         ret = ERROR_BUFFER_OVERFLOW;
@@ -493,7 +508,7 @@ DWORD WINAPI GetAdaptersInfo(PIP_ADAPTER_INFO pAdapterInfo, PULONG pOutBufLen)
           get_interface_indices( TRUE, &table );
         if (table) {
           size = sizeof(IP_ADAPTER_INFO) * table->numIndexes;
-          size += ipAddrTable->dwNumEntries * sizeof(IP_ADDR_STRING); 
+          size += ipAddrTable->dwNumEntries * sizeof(IP_ADDR_STRING);
           if (*pOutBufLen < size) {
             *pOutBufLen = size;
             ret = ERROR_INSUFFICIENT_BUFFER;
@@ -1253,7 +1268,7 @@ ULONG WINAPI DECLSPEC_HOTPATCH GetAdaptersAddresses(ULONG family, ULONG flags, P
     InterfaceIndexTable *table;
     ULONG i, size, dns_server_size = 0, dns_suffix_size, total_size, ret = ERROR_NO_DATA;
 
-    TRACE("(%d, %08x, %p, %p, %p)\n", family, flags, reserved, aa, buflen);
+    TRACE("(%d, %08x, %p, %p, %p, %d)\n", family, flags, reserved, aa, buflen, *buflen);
 
     if (!buflen) return ERROR_INVALID_PARAMETER;
 
@@ -1687,7 +1702,7 @@ DWORD WINAPI GetInterfaceInfo(PIP_INTERFACE_INFO pIfTable, PULONG dwOutBufLen)
 /******************************************************************
  *    GetIpAddrTable (IPHLPAPI.@)
  *
- * Get interface-to-IP address mapping table. 
+ * Get interface-to-IP address mapping table.
  *
  * PARAMS
  *  pIpAddrTable [Out]    buffer for mapping table
@@ -2063,8 +2078,8 @@ BOOL WINAPI GetRTTAndHopCount(IPAddr DestIpAddress, PULONG HopCount, ULONG MaxHo
  *  Failure: error code from winerror.h
  *
  * NOTES
- *  If pdwSize is less than required, the function will return 
- *  ERROR_INSUFFICIENT_BUFFER, and *pdwSize will be set to 
+ *  If pdwSize is less than required, the function will return
+ *  ERROR_INSUFFICIENT_BUFFER, and *pdwSize will be set to
  *  the required byte size.
  *  If bOrder is true, the returned table will be sorted, first by
  *  local address and port number, then by remote address and port
@@ -2130,7 +2145,7 @@ DWORD WINAPI GetExtendedTcpTable(PVOID pTcpTable, PDWORD pdwSize, BOOL bOrder,
  *  Failure: error code from winerror.h
  *
  * NOTES
- *  If pdwSize is less than required, the function will return 
+ *  If pdwSize is less than required, the function will return
  *  ERROR_INSUFFICIENT_BUFFER, and *pdwSize will be set to the
  *  required byte size.
  *  If bOrder is true, the returned table will be sorted, first by
@@ -2276,9 +2291,16 @@ DWORD WINAPI IpRenewAddress(PIP_ADAPTER_INDEX_MAP AdapterInfo)
  */
 DWORD WINAPI NotifyAddrChange(PHANDLE Handle, LPOVERLAPPED overlapped)
 {
+  if (!overlapped)
+  {
+    FIXME("Synchronous method not supported\n");
+    return ERROR_NOT_SUPPORTED;
+  }
   FIXME("(Handle %p, overlapped %p): stub\n", Handle, overlapped);
   if (Handle) *Handle = INVALID_HANDLE_VALUE;
-  if (overlapped) ((IO_STATUS_BLOCK *) overlapped)->u.Status = STATUS_PENDING;
+  ((IO_STATUS_BLOCK *) overlapped)->u.Status = STATUS_PENDING;
+  if (LoadWinsock())
+    pWSASetLastError(WSA_IO_PENDING);
   return ERROR_IO_PENDING;
 }
 
@@ -2301,8 +2323,19 @@ DWORD WINAPI NotifyAddrChange(PHANDLE Handle, LPOVERLAPPED overlapped)
  */
 DWORD WINAPI NotifyRouteChange(PHANDLE Handle, LPOVERLAPPED overlapped)
 {
+  if (!overlapped)
+  {
+    FIXME("Synchronous method not supported\n");
+    return ERROR_NOT_SUPPORTED;
+  }
   FIXME("(Handle %p, overlapped %p): stub\n", Handle, overlapped);
-  return ERROR_NOT_SUPPORTED;
+  if (Handle) *Handle = INVALID_HANDLE_VALUE;
+  ((IO_STATUS_BLOCK *) overlapped)->u.Status = STATUS_PENDING;
+  if (LoadWinsock())
+  {
+    pWSASetLastError(WSA_IO_PENDING);
+  }
+  return ERROR_IO_PENDING;
 }
 
 
